#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu
"""
(c) 2013 - Ronan Delacroix
Sequential files detection lib
:author: Ronan Delacroix & Wolfgang Whoel (cinemaslides)
"""

import re
import os.path
import glob
from . import code as code_utils


class SequentialCandidate(code_utils.SerializableObject):
    """
    SequentialCandidate aims to detect File Sequences in a list of files.
    """

    def __init__(self, args):
        """
        Create the object with a list of files.
        """
        self.type = "SequentialCandidate"
        self.args = sorted(args)
        self.number_of_args = len(self.args)
        self.uuid_re = u'[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}'
        if re.search(self.uuid_re, self.args[0]):
            args_uuid_safe = map(self._mark_uuid, self.args)
        else:
            args_uuid_safe = self.args
        self._check_sequential(args_uuid_safe)

    def _check_sequential(self, args):
        self.composite = ''
        self.ffmpeg_composite = ''
        self.sequence = False
        self.orders = []
        splits = []
        for token in args:
            splits.append(self._numeric_and_non_numeric_particles(os.path.basename(token)))
        for index in range(len(splits[0])):
            column = []
            for row in splits:
                # catch dangling ends of variable-sized split lists
                if index > len(row) - 1:
                    column.append('')
                else:
                    if re.match('\d+', row[index]):
                        numerical = True
                        column.append(row[index])
                    else:
                        numerical = False
                        column.append(row[index])
            if numerical:
                continuous, order, step = self._test_continuity(column)
            else:
                continuous = False
            if continuous:
                self.sequence = True
                self.orders.append(order)
                self.first = int(column[0])
                self.last = int(column[-1])
                self.composite += '[' + str(column[0]) + '-' + str(column[-1]) + ']'
                self.ffmpeg_composite += "%%%sd" % str(len(str(column[-1])))
            else:
                if len(list(set(column))) == 1:
                    self.composite += str(column[0])
                    self.ffmpeg_composite += str(column[0])
                else:
                    self.composite += '[GARBLED]'
                    self.ffmpeg_composite += "*"

    def _numeric_and_non_numeric_particles(self, token):
        splits = re.split('(\D+)', token)
        return splits

    def _test_continuity(self, sequence):
        continuous = False
        continuity_broken = False
        order = ''
        step = 0
        try:
            initial_step = int(sequence[1]) - int(sequence[0])
        except:
            return False, 'None', 0
        for index, element in enumerate(sequence):
            if index == len(sequence) - 1:
                break
            else:
                step = int(sequence[index + 1]) - int(sequence[index])
                if step == 0:
                    break
                elif step == initial_step:
                    continue
                elif step != initial_step:
                    continuity_broken = True
                    break
        if step != 0 and not continuity_broken:
            continuous = True
            if step > 0:
                order = 'ascending by ' + str(step)
            else:
                order = 'descending by ' + str(step)
        return continuous, order, step

    def _mark_uuid(self, string):
        return re.sub(self.uuid_re, '[UUID]', string)

    def __str__(self):
        return self.composite


class SequentialFolder(SequentialCandidate):
    """
    Detects File Sequences in a folder.
    """
    def __init__(self, folder_path):
        self.folder_path = folder_path.rstrip('*').rstrip('/').rstrip('\\')
        args = glob.glob(os.path.join(self.folder_path, '*'))
        args = [os.path.join(self.folder_path, arg) for arg in args]
        SequentialCandidate.__init__(self, args)
        self.type = "SequentialFolder"





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu
"""
(c) 2013 - Ronan Delacroix
Code Utils
:author: Ronan Delacroix
"""

import sys
import importlib
import os
import re
from operator import itemgetter

__singleton_instances = {}


def static_singleton(*args, **kwargs):
    """
    STATIC Singleton Design Pattern Decorator
    Class is initialized with arguments passed into the decorator.

    :Usage: 
     >>> @static_singleton('yop')
        class Bob(Person):
            def __init__(arg1):
                self.info = arg1
            def says(self):
                print self.info
        b1 = Bob #note that we call it by the name of the class, no instance created here, kind of static linking to an instance
        b2 = Bob #here b1 is the same object as b2
        Bob.says() # it will display 'yop'
    """
    
    def __static_singleton_wrapper(cls):        
        if cls not in __singleton_instances:
            __singleton_instances[cls] = cls(*args, **kwargs)

        return __singleton_instances[cls]

    return __static_singleton_wrapper


class cached_property(object):
    """
    Method Descriptor (non-data) for building an attribute on-demand on first use.
    """
    def __init__(self, factory):
        """
        <factory> is called such: factory(instance) to build the attribute.
        """
        self._attr_name = factory.__name__
        self._factory = factory

    def __get__(self, instance, owner):
        # Build the attribute.
        attr = self._factory(instance)

        # Cache the value; hide ourselves.
        setattr(instance, self._attr_name, attr)

        return attr


def import_from_name(module_name):
    globals()[module_name] = __import__(module_name)


class AttributeDict(dict):
    #__getattr__ = dict.__getitem__
    #__setattr__ = dict.__setitem__  #Not using this method as setattr dont work at runtime...

    def __getitem__(self, attr):
        try:
            item = super(AttributeDict, self).__getitem__(attr)
        except KeyError:
            return AttributeDict()
        if isinstance(item, dict) and not isinstance(item, AttributeDict):
            new = AttributeDict(item)
            self.__setitem__(attr, new)
            return new
        return item

    def __getattr__(self, attr):
        if isinstance(self[attr], dict) and not isinstance(self[attr], AttributeDict):
            self[attr] = AttributeDict(self[attr])
            return self[attr]
        if isinstance(self[attr], list):
            res = []
            for f in self[attr]:
                if isinstance(f, dict) and not isinstance(f, AttributeDict):
                    res.append(AttributeDict(f))
                else:
                    res.append(f)
            return res
        return self[attr]

    def __setattr__(self, attr, value):
        self[attr] = value


def safe_dict(obj_or_func, **kwargs):
    """
    Create a dict from any object with all attributes, but not the ones starting with an underscore _
    Useful for objects or function that return an object that have no __dict__ attribute, like psutil functions.
    """
    if callable(obj_or_func):
        res = obj_or_func(**kwargs)
    else:
        res = obj_or_func
    if hasattr(res, '__dict__'):
        return res.__dict__

    attributes = [i for i in dir(res) if not i.startswith('_')]
    out = {}
    for a in attributes:
        val = getattr(res, a)
        if val and not callable(val):
            out[a] = val

    return out


class ProcessException(Exception):
    """
       Process Exception class.
       Displays the PID of the process in its representation.
    """
    def __init__(self, message):
        super(ProcessException, self).__init__(message)

    def __str__(self):
        return self.__repr__()

    def __unicode__(self):
        return self.__repr__()

    def __repr__(self):
        return "%s - Process ID : %s" % (super(ProcessException, self).__str__(), os.getpid())


def get_method_documentation(method):
    """
    This function uses "inspect" to retrieve information about a method.

    Also, if you place comment on the method, method can be docummented with "reStructured Text".

    :param method:    method to describe

    :returns:
        {
            'name'          : <string> - name of the method,
            'friendly_name' : <string> - friendly name of the method,
            'parameters'    : {
                'required'     : [ 'param1', 'param2' ],
                'optionnal'    : {
                    'param3'       : 'default_value3',
                    'param4'       : 'default_value4',
            }, 
            'help'       : {
                'summary'    : <string> - Summary - general description like in the comment,
                'parameters' : {
                    'param1' : 'description',
                    'param2' : 'description',
                }, 
                'return'     : <string> - Can be multiline,
            }
        }
    """
    from inspect import getargspec
    result = {
        'name': method.__name__,
        'friendly_name': ' '.join([name.capitalize() for name in method.__name__.split('_')]),
    }
    arg_specs = getargspec(method)
    arguments = {}

    if not arg_specs.defaults:
        if len(arg_specs.args[1:]) > 0:
            arguments['required'] = list(arg_specs.args[1:])
    else:
        if len(arg_specs.args[1:-(len(arg_specs.defaults))]):
            arguments['required'] = list(arg_specs.args[1:-(len(arg_specs.defaults))])
        arguments['optional'] = {}
        for i in range(len(arg_specs.defaults)):
            arguments['optional'][arg_specs.args[-(len(arg_specs.defaults)) + i]] = arg_specs.defaults[i]
    if arguments != {}:
        result['parameters'] = arguments

    doc = method.__doc__.strip() if method.__doc__ else ''
    if ':' in method.__doc__:
        doc = {'summary': method.__doc__[0:doc.find('  :')].strip()}
        params = re.findall(r":param ([^\s]*): (.*)\n", method.__doc__)
        if len(params) > 0:
            doc['parameters'] = {}
            for param in params:
                doc['parameters'][param[0]] = param[1].strip()

        regex = re.compile(r":returns:(.*)", re.MULTILINE|re.DOTALL)
        returns = regex.search(method.__doc__)
        if returns and returns.group(0):
            doc['return'] = returns.group(0).replace(':returns:', '').replace('\n        ', '\n').strip()
    if doc != '':
        result['help'] = doc
    return result


def get_subclasses(klass):
    assert isinstance(klass, type)
    klasses = klass.__subclasses__()
    klasses2 = klasses
    for y in klasses:
        klasses2 += get_subclasses(y)
    return list(set(klasses2))


class SerializableObject(object):
    """
    Serializable object : allow to export an object as a dict or to fill an object from a dict
    """
    def fill(self, _dict, class_list=None):
        for (key, value) in _dict.items():
            if type(value) in [list, dict, set]:
                value = self.recursive_object_check(value, class_list)
            if key != '_id':
                self.__setattr__(key, value)
        return self

    def recursive_object_check(self, elem, class_list):
        if isinstance(elem, list):
            i = 0
            while i < len(elem):
                if type(elem[i]) in [list, dict, set]:
                    elem[i] = self.recursive_object_check(elem[i], class_list)
                i += 1
            return elem
        elif isinstance(elem, dict):
            if class_list and 'type' in elem and 'uuid' in elem and elem['type'] in [_class.__name__ for _class in class_list]:
                for _class in class_list:
                    if elem['type'] == _class.__name__:
                        obj = _class().fill(elem, class_list=class_list)
                        return obj
            else:
                for (key, value) in elem.items():
                    if type(value) in [list, dict, set]:
                        elem[key] = self.recursive_object_check(value, class_list)

        return elem

    def to_dict(self, dic=None):
        if not dic and dic != {}:
            dic = self.__dict__.copy() # we copy the __dict__ otherwise all the values objects, even strings, will be still referencing "self" ones. That means changing a value in the dict will change value of object, we dont want that.
        for (key, value) in dic.items():

            if isinstance(value, SerializableObject):
                dic[key] = value.to_dict()
            elif isinstance(value, list):
                dic[key] = []
                for f in value:
                    if isinstance(f, SerializableObject):
                        dic[key].append(f.to_dict())
                    else:
                        dic[key].append(f)
            elif isinstance(value, dict):
                dic[key] = self.to_dict(dic=value)
        if '_id' in dic:
            del dic['_id']
        return dic

    def __str__(self):
        return self.__class__.__name__ + "   " + str(self.to_dict())

    def __iter__(self):
        return iter([self])

    def safe_info(self, dic=None):
        """
        Returns public information of the object
        """
        if dic is None and dic != {}:
            dic = self.to_dict()
        output = {}
        for (key, value) in dic.items():
            if key[0] != '_':
                if isinstance(value, SerializableObject):
                    output[key] = value.safe_info()
                elif isinstance(value, dict):
                    output[key] = self.safe_info(dic=value)
                elif isinstance(value, list):
                    output[key] = []
                    for f in value:
                        if isinstance(f, SerializableObject):
                            output[key].append(f.safe_info())
                        elif isinstance(f, dict):
                            output[key].append(self.safe_info(dic=f))
                        else:
                            output[key].append(f)
                else:
                    output[key] = value
        return output


def sort_dictionary_list(dict_list, sort_key):
    """
    sorts a list of dictionaries based on the value of the sort_key

    dict_list - a list of dictionaries
    sort_key - a string that  identifies the key to sort the dictionaries with.

    Test sorting a list of dictionaries:
        >>> sort_dictionary_list([{'b' : 1, 'value' : 2}, {'c' : 2, 'value' : 3}, {'a' : 3, 'value' : 1}], 'value')
        [{'a': 3, 'value': 1}, {'b': 1, 'value': 2}, {'c': 2, 'value': 3}]
    """
    if not dict_list or len(dict_list) == 0:
        return dict_list
    dict_list.sort(key=itemgetter(sort_key))
    return dict_list


def get_app_name():
    mod = sys.modules.get('__main__', sys.modules[__name__])
    if '__file__' in dir(mod):
        script_folder = os.path.dirname(os.path.abspath(mod.__file__))
        app_name = os.path.split(mod.__file__)[1].replace('.pyc', '').replace('.py', '')
    else:
        script_folder = sys.path[0] if sys.path[0] != '' else sys.path[1]
        app_name = os.path.split(script_folder)[1].replace('.pyc', '').replace('.py', '')
    return script_folder, app_name


def lazy_load_module(name):
    """
    Lazy load module function.
    Use:
        lazy_load_module(__name__)
    in a __init__.py package file containing sub modules.

    This allows to import the base module, but access those easily without importing each.
    """
    sys.modules[name] = LazyLoader(name)
    return


class LazyLoader:
    """
    Module Wrapper Class
    See here for wrapping module class : http://stackoverflow.com/questions/2447353/getattr-on-a-module
    Also does lazy loading of sub modules.
    """
    def __init__(self, name):
        self.__module_name = name
        self.__wrapped_module = sys.modules[self.__module_name]
        self.__submodule_imports = {}

    def __getattr__(self, name):
        """
        Lazy loading module
        """
        if name.startswith('__'): #importing needs access to some private reserved __*__ functions...
            return getattr(self.__wrapped_module, name)

        if not self.__submodule_imports.get(name, None):
            self.__submodule_imports[name] = importlib.import_module('.'+name, package=self.__module_name)

        return self.__submodule_imports.get(name, None)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu
"""
(c) 2013 - Ronan Delacroix
Text Utils
:author: Ronan Delacroix
"""
import re
import requests
import os
import os.path


def download(url, folder, force_filename=None, verify=False):
    folder = os.path.abspath(folder)

    if url.startswith('//'):
        url = 'http:'+url
    if force_filename:
        file_name = force_filename
    else:
        file_name = url.split('/')[-1]
        file_name = file_name.split('?')[0]
        file_name = file_name.split('#')[0]

    r = requests.get(url, verify=verify)

    d = r.headers.get('content-disposition', '')
    request_fname = re.findall("filename=(.+)", d)
    if request_fname and not force_filename:
        file_name = request_fname

    path = os.path.abspath(os.path.join(folder, file_name))
    if folder not in path:
        raise Exception("Problem with filepath being not contained in targeted folder (security issue) - %s / %s" %
                        (folder, path))

    with open(path, "wb") as img_file:
        img_file.write(r.content)

    return path


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu
"""
(c) 2013 - Ronan Delacroix
Process Utils
:author: Ronan Delacroix
"""

import os
import sys
import subprocess
import signal
import uuid as UUID
from datetime import datetime
import time
import tempfile
import threading
import re
import shutil
import atexit

import logging
from . import text as text_utils


def restart_program():
    """
    Restarts the current program.
    Note: this function does not return. Any cleanup action (like
    saving data) must be done before calling this function.
    """
    logging.debug("Restarting program...")
    python = sys.executable
    os.execl(python, python, * sys.argv)


def synchronized(lock):
    """
    Synchronization decorator; provide thread-safe locking on a function
    http://code.activestate.com/recipes/465057/
    """
    def wrap(f):
        def synchronize(*args, **kw):
            lock.acquire()
            try:
                return f(*args, **kw)
            finally:
                lock.release()
        return synchronize
    return wrap


def call_repeatedly(func, interval, *args, **kwargs):
    """
    Call a function at interval
    Returns both the thread object and the loop stopper Event.
    """
    main_thead = threading.current_thread()
    stopped = threading.Event()

    def loop():
        while not stopped.wait(interval) and main_thead.is_alive():  # the first call is in `interval` secs
            func(*args, **kwargs)

        return

    timer_thread = threading.Thread(target=loop, daemon=True)
    timer_thread.start()

    atexit.register(stopped.set)

    return timer_thread, stopped.set


def synchronized_limit(lock):
    """
    Synchronization decorator; provide thread-safe locking on a function
    http://code.activestate.com/recipes/465057/
    """
    def wrap(f):
        def synchronize(*args, **kw):
            if lock[1] < 10:
                lock[1] += 1
                lock[0].acquire()
                try:
                    return f(*args, **kw)
                finally:
                    lock[1] -= 1
                    lock[0].release()
            else:
                raise Exception('Too busy')
        return synchronize
    return wrap


def execute(command, return_output=True, log_file=None, log_settings=None, error_logfile=None, timeout=None, line_function=None, poll_timing = 0.01, logger=None, working_folder=None, env=None):
    """
        Execute a program and logs standard output into a file.

        :param return_output:      returns the STDOUT value if True or returns the return code
        :param logfile:            path where log file should be written ( displayed on STDOUT if not set)
        :param error_logfile:      path where error log file should be written ( displayed on STDERR if not set)
        :param timeout:            if set, it will kill the subprocess created when "timeout" seconds is reached. It will then raise an Exception.
        :param line_function:      set it to a "function pointer" for the function to be called each time a new line is written (line passed as a parameter).
        :param poll_timing:        wait time between timeout checks and std output check.

        :returns:   Standard output of the command or if return_output=False, it will give the "return code" of the command
    """

    tmp_log = False
    if log_settings:
        log_folder = log_settings.get('LOG_FOLDER')
    else:
        tmp_log = True
        log_folder = tempfile.mkdtemp()

    if not log_file:
        log_file = os.path.join(log_folder, "commands", "execute-command-logfile-%s.log" % UUID.uuid4())
        try:
            if not os.path.isdir(os.path.join(log_folder, "commands")):
                os.makedirs(os.path.join(log_folder, "commands"))
        except:
            pass
    if not logger:
        logger = logging.getLogger('command_execute')

    logfile_writer = open(log_file, 'a')
    header = "%s - Executing command (timeout=%s) :\n\t%s\n\n\n" % (datetime.now().isoformat(), timeout, command)
    logfile_writer.write(header)
    logfile_writer.flush()

    logfile_reader = open(log_file, 'rb')
    logfile_reader.seek(0, os.SEEK_END)
    logfile_start_position = logfile_reader.tell()

    if error_logfile:
        err_logfile_writer = open(error_logfile, 'a')
    else:
        err_logfile_writer = logfile_writer

    start = datetime.now()
    timeout_string = ""
    if timeout:
        timeout_string = "(timeout=%s)" % timeout
    logger.info(u"Executing command %s :\n\t\t%s" % (timeout_string, command) )

    # We use "exec <command>" as Popen launches a shell, that runs the command.
    # It will transform the child process "sh" into the "command exectable" because of the "exec".
    # Said more accuratly, it won't fork to create launch the command in a sub sub process.
    # Therefore, when you kill the child process, you kill the "command" process and not the unecessary "sh" parent process.
    if sys.platform != 'win32':
        command = u"exec %s" % text_utils.uni(command)
    process = subprocess.Popen(command, stdout=logfile_writer, stderr=err_logfile_writer, bufsize=1, shell=True, cwd=working_folder, env=env)

    while process.poll() == None:
        # In order to avoid unecessary cpu usage, we wait for "poll_timing" seconds ( default: 0.1 sec )
        time.sleep(poll_timing)

        # Timeout check
        if timeout != None:
            now = datetime.now()
            if (now - start).seconds> timeout:
                #process.terminate() ??
                os.kill(process.pid, signal.SIGKILL)
                os.waitpid(-1, os.WNOHANG)
                raise Exception("Command execution timed out (took more than %s seconds...)" % timeout)

        # Line function call:
        #   => if line_function is defined, we call it on each new line of the file.
        if line_function:
            o = text_utils.uni(logfile_reader.readline()).rstrip()
            while o != '':
                line_function(o)
                o = text_utils.uni(logfile_reader.readline()).rstrip()

    if not return_output:
        # Return result code and ensure we have waited for the end of sub process
        return process.wait()

    logfile_reader.seek(logfile_start_position, os.SEEK_SET) #back to the beginning of the file

    res = text_utils.uni(logfile_reader.read())

    try:
        logfile_reader.close()
        logfile_writer.close()
        err_logfile_writer.close()

        if tmp_log:
            shutil.rmtree(log_folder, ignore_errors=True)
    except:
        logger.exception("Error while cleaning after tbx.execute() call.")

    return res


def daemonize(umask=0, work_dir="/", max_fd=1024, redirect="/dev/null"):
    """
    When this function is called, the process is daemonized (by forking + killing its parent).
    It becomes a background task.
    It is useful to release the console.
    """
    if not redirect:
        redirect = "/dev/null"

    if hasattr(os, "devnull"):
        redirect = os.devnull

    try:
        pid = os.fork()
    except OSError as e:
        raise Exception("%s [%d]" % (e.strerror, e.errno))

    # first child
    if pid == 0:
        os.setsid()

        try:
            # Fork a second child.
            pid = os.fork()
        except OSError as e:
            raise Exception("%s [%d]" % (e.strerror, e.errno))

        # The second child.
        if pid == 0:
            os.chdir(work_dir)
            os.umask(umask)
        else:
            # exit first child
            os._exit(0)
    else:
        # Exit parent
        os._exit(0)

    #killing inherited file descriptors
    import resource
    maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
    if maxfd == resource.RLIM_INFINITY:
        maxfd = max_fd

    # close all file descriptors.
    for fd in range(0, maxfd):
        try:
            os.close(fd)
        except OSError:
            # ignored
            pass

    os.open(redirect, os.O_RDWR) # standard input

    # Duplicate standard
    os.dup2(0, 1)			# standard output (1)
    os.dup2(0, 2)			# standard error (2)

    return os.getpid()


def is_running(process):
    s = subprocess.Popen(["ps", "axw"], stdout=subprocess.PIPE)
    for x in s.stdout:
        if re.search(process, x):
            return True
    return False


if __name__ == "__main__":
    import log
    log.configure_logging('process_test')

    COMMAND = """ifconfig {}""".format("-a -v")

    def show_address(line):
        if "inet " in line:
            addr = line.strip().split(' ')[1]
            print("### ADDRESS FOUND : {} ###".format(addr))

    result = execute(COMMAND, line_function=show_address)
    #print(result)



# coding: utf-8
from __future__ import print_function
"""
(c) 2013 - Ronan Delacroix
Logging Utils
:author: Ronan Delacroix
"""
import sys
import os
import socket
import logging
import logging.handlers
from . import code


def configure_logging_to_screen(debug=False):
    level = 'INFO'
    if debug:
        level = 'DEBUG'
    (script_folder, app_name) = code.get_app_name()
    settings = {'LOGGING_LEVEL': level, 'LOGGING_METHODS': ['SCREEN'], 'SCREEN_FORMAT': '%(levelname)s\t| %(message)s'}
    configure_logger(logging.getLogger(), app_name, settings=settings)


def configure_logging(log_name, settings={}, application_name=None, force=False):
    configure_logger(logging.getLogger(), log_name, settings=settings, application_name=application_name, force=force)


def configure_logger(logger, log_name, settings={}, application_name=None, force=False):

    log_level = settings.get('LOGGING_LEVEL', 'DEBUG')
    log_methods = settings.get('LOGGING_METHODS', ['SCREEN', 'FILE', 'SYSLOG'])

    logger.setLevel(log_level)
    
    if not hasattr(logger, 'handlers_added') or force:
        #we first remove all handlers  
        for handler in logger.handlers:
            logger.removeHandler(handler)

        #make handlers, that write the logs to screen, file, syslog
        if 'SCREEN' in log_methods:
            add_screen_logging(logger, settings)

        if 'SYSLOG' in log_methods:# and ('SysLogHandler' in dir(logging)):
            add_syslog_logging(logger, log_name, settings)

        if 'FILE' in log_methods:
            add_file_logging(logger, log_name, application_name, settings)

        if 'MONGO' in log_methods:
            add_mongo_logging(logger, log_name, application_name, settings)

        logger.propagate = True
    
        logger.handlers_added = True


def add_screen_logging(logger, settings={}):
    screen_format = settings.get('SCREEN_FORMAT', '%(levelname)s\t| %(message)s')
    write_to_screen_handler = logging.StreamHandler()
    screen_formatter = logging.Formatter(screen_format, '%Y-%m-%dT%H:%M:%S')
    write_to_screen_handler.setFormatter(screen_formatter)
    logger.addHandler(write_to_screen_handler)


def add_syslog_logging(logger, log_name, settings={}):
    #guessing syslog address
    syslog_address = settings.get('LOGGING_SYSLOG_ADDRESS', None)
    if not syslog_address:
        if sys.platform == 'darwin':
            syslog_address = '/var/run/syslog'
        else:
            syslog_address = '/dev/log'
    syslog_format = settings.get('SYSLOG_FORMAT', log_name+': [%(filename)s:%(funcName)s:%(lineno)d]\t%(levelname)s - %(message).1900s')
    write_to_syslog_handler = logging.handlers.SysLogHandler(address=syslog_address)
    syslog_formatter = logging.Formatter(syslog_format, '%Y-%m-%dT%H:%M:%S')
    write_to_syslog_handler.setFormatter(syslog_formatter)
    logger.addHandler(write_to_syslog_handler)


def add_file_logging(logger, log_name, application_name, settings={}):
    (script_folder, app_name) = code.get_app_name()
    if not application_name:
        application_name = app_name

    log_folder = settings.get('LOGGING_FOLDER', None)
    if not log_folder:
        log_folder = os.path.join(script_folder, 'log')

    log_folder = log_folder.replace('<app_name>', application_name).replace('<name>', log_name)
    log_folder = os.path.abspath(log_folder)

    if not os.path.isdir(log_folder):
        try:
            os.makedirs(log_folder)
        except:
            print("Warning : permission denied to log in folder '%s'. Will attempt to log in '%s'" %
                  (log_folder, os.path.join(script_folder, 'log')))
            log_folder = os.path.join(script_folder, 'log')

    if not os.path.isdir(log_folder):
        try:
            os.makedirs(log_folder)
        except:
            print("Impossible to log with FILE handler to %s either, abandoning file logging." % log_folder)
            return

    if not os.access(log_folder, os.W_OK):
        print("Warning : permission denied to log in folder '%s'. Will attempt to log in '%s'" %
              (log_folder, os.path.join(script_folder, 'log')))
        log_folder = os.path.join(script_folder, 'log')
        try:
            os.makedirs(log_folder)
        except:
            print("Impossible to log with FILE handler to %s either, abandoning file logging." % log_folder)
            return

    file_format = settings.get('FILE_FORMAT', None)

    log_file = os.path.join(log_folder, log_name + ".txt")

    add_logging_file_handler(logger, log_file, file_format)


def add_logging_file_handler(logger, log_file, format=None):
    if not format:
        format = '[%(asctime)s] [%(filename)s:%(funcName)s:%(lineno)d]\t%(levelname)s - %(message)s'

    write_to_file_handler = logging.handlers.RotatingFileHandler(log_file, maxBytes=10000000, backupCount=10)
    file_formatter = logging.Formatter(format, '%Y-%m-%dT%H:%M:%S')
    write_to_file_handler.setFormatter(file_formatter)
    logger.addHandler(write_to_file_handler)


def add_mongo_logging(logger, log_name, application_name, settings={}):
    (script_folder, app_name) = code.get_app_name()
    if not application_name:
        application_name = app_name

    try:
        import log4mongo.handlers
    except ImportError:
        print("Impossible to log with MONGO handler as log4mongo library is not available.", file=sys.stderr)
        return

    class MyMongoFormatter(log4mongo.handlers.MongoFormatter):
        def format(self, record):
            document = super(MyMongoFormatter, self).format(record)
            del document['threadName']
            del document['thread']
            del document['loggerName']
            del document['module']
            del document['method']
            document['log_name'] = log_name
            document['hostname'] = socket.gethostname()
            return document

    mongo_handler_class = log4mongo.handlers.MongoHandler
    mongo_handler_args = {
        'host': settings.get('LOGGING_MONGO_HOST', "localhost"),
        'port': settings.get('LOGGING_MONGO_PORT', 27017),
        'database_name': settings.get('LOGGING_MONGO_DATABASE', application_name),
        'collection': settings.get('LOGGING_MONGO_COLLECTION', log_name+"_logs"),
        'capped': settings.get('LOGGING_MONGO_CAPPED', True),
        'capped_max': settings.get('LOGGING_MONGO_CAPPED_MAX', 100000),
        'capped_size': settings.get('LOGGING_MONGO_CAPPED_SIZE', 10000000),
        'formatter' : MyMongoFormatter()
    }
    if settings.get('LOGGING_MONGO_BUFFER_SIZE', False):
        mongo_handler_class = log4mongo.handlers.BufferedMongoHandler
        mongo_handler_args.update({
            'buffer_size': settings.get('LOGGING_MONGO_BUFFER_SIZE', 20),
            'buffer_early_flush_level': settings.get('LOGGING_MONGO_BUFFER_FLUSH_LEVEL', logging.CRITICAL),
            'buffer_periodical_flush_timing': settings.get('LOGGING_MONGO_BUFFER_FLUSH_TIMER', 5.0)
        })

    #class MongoFilter(logging.Filter):
    #    def filter(self, record):
    #        record.log_name = log_name
    #        record.hostname = socket.gethostname()
    #        return True
    #
    #logger.addFilter(MongoFilter())

    log4mongo_handler = mongo_handler_class(**mongo_handler_args)

    logger.addHandler(log4mongo_handler)

